import type { Container } from "./Container";
import type { IDistance } from "./Interfaces/IDistance";
import type { IParticleValueAnimation } from "./Interfaces/IParticleValueAnimation";
import type { ICoordinates, ICoordinates3d } from "./Interfaces/ICoordinates";
import type { IHsl, IRgb } from "./Interfaces/Colors";
import type { IShapeValues } from "../Options/Interfaces/Particles/Shape/IShapeValues";
import type { IBubbleParticleData } from "./Interfaces/IBubbleParticleData";
import type { IParticle } from "./Interfaces/IParticle";
import type { IParticles } from "../Options/Interfaces/Particles/IParticles";
import { Particles } from "../Options/Classes/Particles/Particles";
import { MoveDirection, MoveDirectionAlt, ShapeType } from "../Enums";
import type { RecursivePartial } from "../Types";
import type { IDelta } from "./Interfaces/IDelta";
import type { ILink } from "./Interfaces/ILink";
import type { IParticleLoops } from "./Interfaces/IParticleLoops";
import type { IParticleInfection } from "./Interfaces/IParticleInfection";
import type { IParticleHslAnimation } from "./Interfaces/IParticleHslAnimation";
import type { Stroke } from "../Options/Classes/Particles/Stroke";
import type { IParticleLife } from "./Interfaces/IParticleLife";
import type { IParticleSpin } from "./Interfaces/IParticleSpin";
import { Vector } from "./Particle/Vector";
import { Vector3d } from "./Particle/Vector3d";
export declare class Particle implements IParticle {
    readonly id: number;
    readonly container: Container;
    readonly group?: string | undefined;
    destroyed: boolean;
    misplaced: boolean;
    spawning: boolean;
    lastNoiseTime: number;
    zIndexFactor: number;
    splitCount: number;
    unbreakable: boolean;
    readonly noiseDelay: number;
    readonly sides: number;
    readonly strokeWidth: number;
    readonly options: Particles;
    links: ILink[];
    randomIndexData?: number;
    linksDistance?: number;
    attractDistance?: number;
    linksWidth?: number;
    moveSpeed: number;
    sizeAnimationSpeed?: number;
    orbitRadius?: number;
    orbitRotation?: number;
    close: boolean;
    fill: boolean;
    readonly direction: MoveDirection | keyof typeof MoveDirection | MoveDirectionAlt;
    readonly infection: IParticleInfection;
    readonly life: IParticleLife;
    readonly loops: IParticleLoops;
    readonly spin?: IParticleSpin;
    readonly stroke: Stroke;
    readonly position: Vector3d;
    readonly offset: Vector;
    readonly shadowColor: IRgb | undefined;
    readonly color?: IParticleHslAnimation;
    readonly maxDistance: Partial<IDistance>;
    readonly opacity: IParticleValueAnimation<number>;
    readonly rotate: IParticleValueAnimation<number>;
    readonly size: IParticleValueAnimation<number>;
    readonly strokeColor?: IParticleHslAnimation;
    readonly orbitColor?: IHsl;
    readonly velocity: Vector;
    readonly shape: ShapeType | string;
    readonly initialPosition: Vector3d;
    readonly initialVelocity: Vector;
    readonly shapeData?: IShapeValues;
    readonly bubble: IBubbleParticleData;
    constructor(id: number, container: Container, position?: ICoordinates, overrideOptions?: RecursivePartial<IParticles>, group?: string | undefined);
    draw(delta: IDelta): void;
    getPosition(): ICoordinates3d;
    getRadius(): number;
    getFillColor(): IHsl | undefined;
    getStrokeColor(): IHsl | undefined;
    destroy(override?: boolean): void;
    reset(): void;
    private setColorAnimation;
    private calcPosition;
    fixPositionInCanvas(pos: ICoordinates3d): void;
    private getOutCanvasFix;
    private checkOverlap;
    private calculateVelocity;
    private split;
    private loadShapeData;
}
