import { checkDestroy, clamp } from "../Utils";
export class SizeUpdater {
    constructor(container) {
        this.container = container;
    }
    isEnabled(particle) {
        const sizeOpt = particle.options.size;
        const sizeAnim = sizeOpt.animation;
        return (!particle.destroyed &&
            !particle.spawning &&
            sizeAnim.enable &&
            (sizeAnim.count <= 0 || particle.loops.size < sizeAnim.count));
    }
    update(particle, delta) {
        var _a;
        const container = this.container;
        const sizeOpt = particle.options.size;
        const sizeAnim = sizeOpt.animation;
        const sizeVelocity = ((_a = particle.size.velocity) !== null && _a !== void 0 ? _a : 0) * delta.factor;
        const value = sizeOpt.value;
        const minValue = (typeof value === "number" ? value : value.min) * container.retina.pixelRatio;
        const maxValue = (typeof value === "number" ? value : value.max) * container.retina.pixelRatio;
        if (!this.isEnabled(particle)) {
            return;
        }
        switch (particle.size.status) {
            case 0:
                if (particle.size.value >= maxValue) {
                    particle.size.status = 1;
                    particle.loops.size++;
                }
                else {
                    particle.size.value += sizeVelocity;
                }
                break;
            case 1:
                if (particle.size.value <= minValue) {
                    particle.size.status = 0;
                    particle.loops.size++;
                }
                else {
                    particle.size.value -= sizeVelocity;
                }
        }
        checkDestroy(particle, sizeAnim.destroy, particle.size.value, minValue, maxValue);
        if (!particle.destroyed) {
            particle.size.value = clamp(particle.size.value, minValue, maxValue);
        }
    }
}
