const plugins = [];
const interactorsInitializers = [];
const updatersInitializers = [];
const interactors = new Map();
const updaters = new Map();
const presets = new Map();
const drawers = new Map();
const noiseGenerators = new Map();
export class Plugins {
    static getPlugin(plugin) {
        return plugins.find((t) => t.id === plugin);
    }
    static addPlugin(plugin) {
        if (!Plugins.getPlugin(plugin.id)) {
            plugins.push(plugin);
        }
    }
    static getAvailablePlugins(container) {
        const res = new Map();
        for (const plugin of plugins) {
            if (!plugin.needsPlugin(container.options)) {
                continue;
            }
            res.set(plugin.id, plugin.getPlugin(container));
        }
        return res;
    }
    static loadOptions(options, sourceOptions) {
        for (const plugin of plugins) {
            plugin.loadOptions(options, sourceOptions);
        }
    }
    static getPreset(preset) {
        return presets.get(preset);
    }
    static addPreset(presetKey, options) {
        if (!Plugins.getPreset(presetKey)) {
            presets.set(presetKey, options);
        }
    }
    static addShapeDrawer(type, drawer) {
        if (!Plugins.getShapeDrawer(type)) {
            drawers.set(type, drawer);
        }
    }
    static getShapeDrawer(type) {
        return drawers.get(type);
    }
    static getSupportedShapes() {
        return drawers.keys();
    }
    static getNoiseGenerator(type) {
        return noiseGenerators.get(type);
    }
    static addNoiseGenerator(type, noiseGenerator) {
        if (!Plugins.getNoiseGenerator(type)) {
            noiseGenerators.set(type, noiseGenerator);
        }
    }
    static getInteractors(container) {
        let res = interactors.get(container);
        if (!res) {
            res = interactorsInitializers.map((t) => t(container));
            interactors.set(container, res);
        }
        return res;
    }
    static addInteractor(initInteractor) {
        interactorsInitializers.push(initInteractor);
    }
    static getUpdaters(container) {
        let res = updaters.get(container);
        if (!res) {
            res = updatersInitializers.map((t) => t(container));
            updaters.set(container, res);
        }
        return res;
    }
    static addParticleUpdater(initUpdater) {
        updatersInitializers.push(initUpdater);
    }
}
